// -*- C++ -*-
ACE_BEGIN_VERSIONED_NAMESPACE_DECL

ACE_INLINE
ACE_SOCK_CODgram::ACE_SOCK_CODgram ()
{
  ACE_TRACE ("ACE_SOCK_CODgram::ACE_SOCK_CODgram");
}

ACE_INLINE
ACE_SOCK_CODgram::~ACE_SOCK_CODgram ()
{
  ACE_TRACE ("ACE_SOCK_CODgram::~ACE_SOCK_CODgram");
}

ACE_END_VERSIONED_NAMESPACE_DECL
