/* -*- C++ -*- */
// The following configuration file is designed to work for VxWorks
// 22.03 platforms using one of these compilers:
// 1) The llvm compiler that is shipped with VxWorks 22.03

#ifndef ACE_CONFIG_VXWORKS_22_03_H
#define ACE_CONFIG_VXWORKS_22_03_H
#include /**/ "ace/pre.h"

#include "ace/config-vxworks21.03.h"

#include /**/ "ace/post.h"
#endif /* ACE_CONFIG_VXWORKS_22_03_H */
