
// -*- C++ -*-
// This is file was automatically generated by $ACE_ROOT/bin/make_release.py

#define ACE_MAJOR_VERSION 8
#define ACE_MINOR_VERSION 0
#define ACE_MICRO_VERSION 5
#define ACE_VERSION "8.0.5"
#define ACE_VERSION_CODE 0x80005
#if !defined (ACE_PATCH)
# define ACE_PATCH ""
#endif
#define ACE_MAKE_VERSION_CODE(a,b,c) (((a) << 16) + ((b) << 8) + (c))
