// -*- C++ -*-

//=============================================================================
/**
 *  @file    os_wordexp.h
 *
 *  word-expansion types
 *
 *  @author Don Hinton <dhinton@dresystems.com>
 *  @author This code was originally in various places including ace/OS.h.
 */
//=============================================================================

#ifndef ACE_OS_INCLUDE_OS_WORDEXP_H
#define ACE_OS_INCLUDE_OS_WORDEXP_H

#include /**/ "ace/pre.h"

#include /**/ "ace/config-all.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#include "ace/os_include/os_stddef.h" // size_t

#include /**/ <wordexp.h>

#include /**/ "ace/post.h"
#endif /* ACE_OS_INCLUDE_OS_WORDEXP_H */
